import json

import bs4
import numpy as np
import pandas as pd
import requests

# from bs4 import BeautifulSoup


class Scraper:
    """ Main class for scraping prisjakt """

    base_url = "https://www.prisjakt.no/"

    def get_sellers_and_prices_of_product_list(
        self, products: pd.Series
    ) -> pd.DataFrame:  # pragma nocover
        """Get a full table of all sellers and prices for a list of products

        Args:
            products: A list of the product numbers (int)

        Returns:
            A dataframe with all products, sellers and prices
        """

        out = pd.DataFrame()
        for product in products:
            try:
                out = out.append(self.get_sellers_and_prices_of_product(product))
            except:
                pass
        return out

    def get_all_products_from_category(
        self, category: str, no_pages: int = 10, prods_per_site: int = 36
    ) -> pd.DataFrame:
        """Get all products from a category

        Args:
            category: Category name from url. Example: 'mobiltelefoner' which points to 'prisjakt.no/c/mobiltelefoner'
            no_pages: Amount of pages in the category to scrape
            prods_per_site: Products per site

        Returns:
            A dataframe with columns "Product name" and "Product number"
        """

        category_url = self.base_url + "c/" + category

        out = pd.DataFrame()
        for page_no in range(0, no_pages):
            soup = self._get_soup(category_url + f"?offset={prods_per_site*page_no}")
            out = out.append(self.get_products_from_page(soup))
            out["category"] = category
            out = out.astype({"product_name": object, "product_number": np.int64})
        return out

    def get_products_from_page(self, soup: bs4.BeautifulSoup) -> pd.DataFrame:
        """Get products from a single page

        Args:
            soup: A bs4 soup of given page

        Return:
            A dataframe with columns "Product name" and "Product number"
        """
        prods = soup.select("a[class*='ProductLink']")
        return pd.DataFrame(
            {
                "product_name": [prod.attrs["aria-label"] for prod in prods],
                "product_number": [
                    prod.attrs["href"][prod.attrs["href"].index("=") + 1 :]
                    for prod in prods
                ],
            }
        )

    def get_sellers_and_prices_of_product(self, product_number: str) -> pd.DataFrame:
        """Get sellers and prices of product

        Args:
            product_number: Product number from url. Example: '5172022' which points to 'prisjakt.no/product.php?p=5172022'

        Returns:
            A dataframe with information on sellers and prices
        """

        soup = self._get_soup(self.base_url + "product.php?p=" + product_number)

        info_raw = None
        for script in soup.find_all("script"):
            try:
                if script["data-script"] == "globals":
                    info_raw = script
                    break
            except:
                pass

        if info_raw:
            info_split = info_raw.string.split(',{"__typename":"Price",')[1:]
            info_split = [seller for seller in info_split if "variants" in seller[-30:]]

            out = pd.DataFrame()
            for info in info_split:
                temp_df = pd.DataFrame()
                current = json.loads("{" + info)
                try:
                    temp_df["price_id"] = [current["id"]]
                    temp_df["seller_product_name"] = [current["name"]]
                    temp_df["stock_status"] = [current["stock"]["status"]]
                    temp_df["price_incl_shipping"] = [current["price"]["inclShipping"]]
                    temp_df["price_excl_shipping"] = [current["price"]["exclShipping"]]
                    temp_df["seller_id"] = [current["store"]["id"]]
                    temp_df["seller_name"] = [current["store"]["name"]]
                    temp_df["seller_rating"] = [
                        current["store"]["userReviewSummary"]["rating"]
                    ]
                    out = out.append(temp_df)
                except:
                    pass

            out["product_number"] = product_number
            out = out.astype(
                {
                    "price_id": np.int64,
                    "seller_product_name": object,
                    "stock_status": object,
                    "price_incl_shipping": np.float64,
                    "price_excl_shipping": np.float64,
                    "seller_id": np.int64,
                    "seller_name": object,
                    "seller_rating": np.float64,
                    "product_number": np.int64,
                }
            )

            return out
        else:
            return None

    def _get_soup(self, url: str, parser: str = "html.parser") -> bs4.BeautifulSoup:
        """ Make request and parse to bs4 soup """
        try:
            page = requests.get(url)
            print(page.content.decode("unicode-escape"))
            soup = bs4.BeautifulSoup(page.content.decode("unicode-escape"), parser)
            return soup
        except:
            raise



from unittest import TestCase
from unittest.mock import patch

import bs4
import pandas as pd

from pj_scraper.scraper import Scraper


class TestScraper(TestCase):
    def test_dummy(self):
        assert True

    @patch("requests.get")
    @patch("bs4.BeautifulSoup")
    def test_get_soup(self, mock_bs4, mock_req):
        """ Test initialization of soup """
        s = Scraper()
        s._get_soup("some_url")
        mock_req.assert_called_with("some_url")
        mock_bs4.assert_called()

    def test_get_products(self):
        """ Test that correct info is gathered from a page """
        dummy_request_content = '<a aria-label="Apple iPhone 12 64GB" class="ProductLink-sc-882dpj-1 kHbpTA" href="/product.php?p=5594641"><a aria-label="Apple iPhone 12 64GB" class="ProductLink-sc-882dpj-1 kHbpTA" href="/product.php?p=5594641">'
        dummy_soup = bs4.BeautifulSoup(dummy_request_content)

        s = Scraper()
        products = s.get_products_from_page(dummy_soup)

        self.assertEqual(products.loc[0, "product_name"], "Apple iPhone 12 64GB")
        self.assertEqual(products.loc[1, "product_number"], "5594641")

    @patch("pj_scraper.scraper.Scraper._get_soup")
    @patch("pj_scraper.scraper.Scraper.get_products_from_page")
    def test_all_prods_from_category(self, mock_get_prods, mock_get_soup):
        """ """

        s = Scraper()
        cat = "some_category"

        mock_get_prods.return_value = pd.DataFrame(
            {"product_name": ["product"], "product_number": ["1234"]}
        )
        prods = s.get_all_products_from_category(cat, no_pages=2)
        self.assertEqual(len(prods), 2)
        self.assertEqual(prods.iloc[0]["category"], cat)

    @patch("pj_scraper.scraper.Scraper._get_soup")
    def test_get_sellers_and_prices_of_product(self, mock_get_soup):

        dummy_response = '<script data-script="foo"></script><script data-script="globals">,{"__typename":"Price","id":"4428260864","name":"ASUS ROG STRIX Z590-F GAMING WIFI ATX LGA1200 Intel Z590","type":"normal","externalUri":null,"stock":{"status":"in_stock","statusText":"200 p\xE5 lager"},"deliveryOptions":[],"authorizedBy":[],"price":{"inclShipping":null,"exclShipping":3500,"originalCurrency":"NOK","originalCurrencyExclShipping":"3500"},"store":{"id":37622,"name":"Invensys","featured":false,"hasLogo":false,"logo":null,"pathName":"/shop.php?f=37622","providedByStore":{"generalInformation":null},"userReviewSummary":{"rating":null,"count":0,"countTotal":0},"market":"primary","marketplace":false,"countryCode":"no","primaryMarket":"no","currency":"NOK"},"brandRecommendedRetailer":false,"alternativePrices":[{"__typename":"AlternativePrice","id":"4428260864","name":"ASUS ROG STRIX Z590-F GAMING WIFI ATX LGA1200 Intel Z590","type":"normal","externalUri":null,"stock":{"status":"in_stock","statusText":"200 p\xE5 lager"},"price":{"exclShipping":3500,"inclShipping":null,"originalCurrency":"NOK","originalCurrencyExclShipping":"3500"}},{"__typename":"AlternativePrice","id":"4184620430","name":"ASUS ROG STRIX Z590-F GAMING WIFI (ATX, Z590, LGA 1200)","type":null,"externalUri":null,"stock":{"status":"in_stock","statusText":"4 p\xE5 lager"},"price":{"exclShipping":3795,"inclShipping":null,"originalCurrency":"NOK","originalCurrencyExclShipping":null}}],"variants":null},{"__typename":"Price","id":"4428260864","name":"ASUS ROG STRIX Z590-F GAMING WIFI ATX LGA1200 Intel Z590","type":"normal","externalUri":null,"stock":{"status":"in_stock","statusText":"200 p\xE5 lager"},"deliveryOptions":[],"authorizedBy":[],"price":{"inclShipping":null,"exclShipping":3500,"originalCurrency":"NOK","originalCurrencyExclShipping":"3500"},"store":{"id":37622,"name":"Invensys","featured":false,"hasLogo":false,"logo":null,"pathName":"/shop.php?f=37622","providedByStore":{"generalInformation":null},"userReviewSummary":{"rating":null,"count":0,"countTotal":0},"market":"primary","marketplace":false,"countryCode":"no","primaryMarket":"no","currency":"NOK"},"brandRecommendedRetailer":false,"alternativePrices":[{"__typename":"AlternativePrice","id":"4428260864","name":"ASUS ROG STRIX Z590-F GAMING WIFI ATX LGA1200 Intel Z590","type":"normal","externalUri":null,"stock":{"status":"in_stock","statusText":"200 p\xE5 lager"},"price":{"exclShipping":3500,"inclShipping":null,"originalCurrency":"NOK","originalCurrencyExclShipping":"3500"}},{"__typename":"AlternativePrice","id":"4184620430","name":"ASUS ROG STRIX Z590-F GAMING WIFI (ATX, Z590, LGA 1200)","type":null,"externalUri":null,"stock":{"status":"in_stock","statusText":"4 p\xE5 lager"},"price":{"exclShipping":3795,"inclShipping":null,"originalCurrency":"NOK","originalCurrencyExclShipping":null}}],"variants":null}</script>'
        mock_get_soup.return_value = bs4.BeautifulSoup(dummy_response)

        s = Scraper()
        prices = s.get_sellers_and_prices_of_product("1234")
        self.assertEqual(len(prices), 2)
        self.assertEqual(prices.iloc[0]["price_excl_shipping"], 3500)



from .scraper import Scraper


def main():
    s = Scraper()
    products = s.get_all_products_from_category("smartklokker", no_pages=1)
    sellers_and_prices = s.get_sellers_and_prices_of_product_list(
        products["Product number"]
    )
    print(sellers_and_prices.head())



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

import io
import re
from glob import glob
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import splitext

from setuptools import find_packages
from setuptools import setup


def read(*names, **kwargs):
    with io.open(
        join(dirname(__file__), *names), encoding=kwargs.get("encoding", "utf8")
    ) as fh:
        return fh.read()


setup(
    name="pj-scraper",
    use_scm_version={
        "local_scheme": "dirty-tag",
        "write_to": "src/pj_scraper/_version.py",
        "fallback_version": "0.0.0",
    },
    license="MIT",
    description="none",
    long_description="%s\n%s"
    % (
        re.compile("^.. start-badges.*^.. end-badges", re.M | re.S).sub(
            "", read("README.rst")
        ),
        re.sub(":[a-z]+:`~?(.*?)`", r"``\1``", read("CHANGELOG.rst")),
    ),
    author="Mattias Hornum",
    author_email="mattiashornum@gmail.com",
    url="https://github.com/mdhor/pj-scraper",
    packages=find_packages("src"),
    package_dir={"": "src"},
    py_modules=[splitext(basename(path))[0] for path in glob("src/*.py")],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
        "Operating System :: POSIX",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Utilities",
    ],
    project_urls={
        "Changelog": "https://github.com/mdhor/pj-scraper/blob/master/CHANGELOG.rst",
        "Issue Tracker": "https://github.com/mdhor/pj-scraper/issues",
    },
    keywords=[],
    python_requires=">=3.6",
    install_requires=["pandas", "beautifulsoup4", "requests", "openpyxl"],
    extras_require={},
    setup_requires=[
        "setuptools_scm",
    ],
    entry_points={
        "console_scripts": [
            "pj-scraper = pj_scraper.cli:main",
        ]
    },
)



from pkg_resources import DistributionNotFound
from pkg_resources import get_distribution

try:
    __version__ = get_distribution("pj_scraper").version
except DistributionNotFound:
    from setuptools_scm import get_version

    __version__ = get_version(root="..", relative_to=__file__)
